@@ -25,7 +25,7 @@ from gazpacho.choice import enum_to_string, flags_to_string
 from gazpacho.loader import tags
 from gazpacho.placeholder import Placeholder
 from gazpacho.properties import prop_registry
-from gazpacho.widget import Widget
+from gazpacho.widget import Gadget
 
 def write_xml(file, xml_node, indent=0, indent_increase=4):
     if xml_node.nodeType == xml_node.TEXT_NODE:
@@ -36,14 +36,14 @@ def write_xml(file, xml_node, indent=0, indent_increase=4):
         return
 
     file.write(' '*indent)
-    
+
     file.write('<%s' % xml_node.tagName)
     if len(xml_node.attributes) > 0:
         attr_string = ' '.join(['%s="%s"' % (n, v)
                                     for n, v in xml_node.attributes.items()])
         file.write(' ' + attr_string)
 
-    children = [a for a in xml_node.childNodes 
+    children = [a for a in xml_node.childNodes
                     if a.nodeType != a.ATTRIBUTE_NODE]
     if children:
         has_text_child = False
@@ -73,16 +73,18 @@ class XMLWriter:
             document = dom.createDocument(None, None, None)
         self._doc = document
         self._project = project
-        
-    def write(self, path, widgets, uim):
-        self._write_root(widgets, uim)
-        self._write_gazpacho_document(path, self._doc.documentElement)
+
+    def write(self, path, widgets, sizegroups, uim):
+        root = self._write_root(widgets)
+        self._write_sizegroups(root, sizegroups)
+        self._write_widgets(root, widgets, uim)
+        self._write_gazpacho_document(path, root)
 
     def _write_header(self, f, dtd_url):
         f.write("""<?xml version="1.0" standalone="no"?> <!--*- mode: xml -*-->
 <!DOCTYPE glade-interface SYSTEM "%s">
 """ % dtd_url)
-        
+
     def _write_gazpacho_document(self, path, root):
         f = file(path, 'w')
         self._write_header(f, "http://gazpacho.sicem.biz/gazpacho-0.1.dtd")
@@ -91,7 +93,7 @@ class XMLWriter:
 
     # FIXME: Should this really be exported
     write_node = _write_gazpacho_document
-    
+
     def _write_libglade_document(self, path, root):
         f = file(path, 'w')
         self._write_header(f, "http://glade.gnome.org/glade-2.0.dtd")
@@ -100,102 +102,118 @@ class XMLWriter:
         for node in root.childNodes:
             write_xml(f, node, indent_increase=2)
         f.write('\n</glade-interface>\n')
-        
+
         f.close()
-        
-    def serialize_node(self, widget):
+
+    def serialize_node(self, gadget):
         element = self._doc.createElement(tags.XML_TAG_PROJECT)
         root = self._doc.appendChild(element)
 
         # FIXME: Requirements? see _write_root
-        
+
         # save the UI Manager if needed
-        widget.project.uim.save_widget(widget, root, self._doc)
-        root.appendChild(self._write_widget(widget))
+        gadget.project.uim.save_gadget(gadget, root, self._doc)
+        root.appendChild(self._write_widget(gadget))
 
         return self._doc.documentElement
-        
-    def serialize(self, widget):
+
+    def serialize(self, gadget):
         fp = cStringIO.StringIO()
-        node = self.serialize_node(widget)
+        node = self.serialize_node(gadget)
         write_xml(fp, node)
-        
+
         fp.seek(0)
-        
+
         return fp.read()
 
-    def serialize_widgets(self, widgets, uim):
+    def serialize_widgets(self, widgets, sizegroups, uim):
         fp = cStringIO.StringIO()
-        write_xml(fp, self._write_root(widgets, uim))
+        root = self._write_root(widgets)
+        self._write_sizegroups(root, sizegroups)
+        self._write_widgets(root, widgets, uim)
+        write_xml(fp, root)
         fp.seek(0)
-        
+
         return fp.read()
 
-    def _get_requirements(self, widgets):
-        # check what modules are the widgets using
+    def _get_requirements(self, gadgets):
+        # check what modules are the gadgets using
         # Not implemented
         return []
-    
-    def _write_root(self, widgets, uim):
+
+    def _write_root(self, gadgets):
         project_node = self._doc.createElement(tags.XML_TAG_PROJECT)
         node = self._doc.appendChild(project_node)
 
-        for module in self._get_requirements(widgets):
+        for module in self._get_requirements(gadgets):
             n = self._doc.createElement(tags.XML_TAG_REQUIRES)
             n.setAttribute(tags.XML_TAG_LIB, module)
             node.appendChild(n)
 
-        self._write_widgets(node, widgets, uim)
-
         return node
-    
-    def _write_widgets(self, node, widgets, uim):
 
+    def _write_sizegroups(self, root, sizegroups):
+        for sizegroup in sizegroups:
+            node = self._doc.createElement(tags.XML_TAG_WIDGET)
+            root.appendChild(node)
+            node.setAttribute(tags.XML_TAG_CLASS, 'GtkSizeGroup')
+            node.setAttribute(tags.XML_TAG_ID, sizegroup.name)
+
+            prop_node = self._doc.createElement(tags.XML_TAG_PROPERTY)
+            node.appendChild(prop_node)
+            prop_node.setAttribute(tags.XML_TAG_NAME, 'mode')
+            text = self._doc.createTextNode(enum_to_string(
+                sizegroup.mode, enum=gtk.SizeGroupMode))
+            prop_node.appendChild(text)
+
+    def _write_widgets(self, node, widgets, uim):
         # Append uimanager
-        ui_widgets = [Widget.from_widget(w) 
+        ui_widgets = [Gadget.from_widget(w)
                           for w in widgets
                                if isinstance(w, (gtk.Toolbar,
                                                  gtk.MenuBar))]
-        if ui_widgets:
-            ui_node = uim.save(self._doc, ui_widgets)
-            if ui_node:
-                node.appendChild(ui_node)
-        
+        ui_node = uim.save(self._doc, ui_widgets)
+        if ui_node:
+            node.appendChild(ui_node)
+
         # Append toplevel widgets. Each widget then takes care of
         # appending its children
         for widget in widgets:
-            gwidget = Widget.from_widget(widget)
-            if not gwidget.is_toplevel():
+            gadget = Gadget.from_widget(widget)
+            if (gadget is None or
+                not gadget.is_toplevel()):
                 continue
-            
-            wnode = self._write_widget(gwidget)
-            node.appendChild(wnode)
-    
-    def _write_widget(self, widget):
-        """Serializes this widget into a XML node and returns this node"""
 
-        widget.maintain_gtk_properties = True
-        
-        widget.adaptor.save(widget.project.context, widget)
+            wnode = self._write_widget(gadget)
+            node.appendChild(wnode)
+
+        return node
+
+    def _write_widget(self, gadget):
+        """Serializes this gadget into a XML node and returns this node"""
+
+        gadget.maintain_gtk_properties = True
+
+        gadget.adaptor.save(gadget.project.context, gadget)
 
         # otherwise use the default saver
-        node = self._write_basic_information(widget)
-        
-        self._write_properties(widget, node, child=False)
-        self._write_signals(widget, node)
-        
+        node = self._write_basic_information(gadget)
+
+        self._write_properties(node, gadget, child=False)
+        self._write_signals(node, gadget.get_all_signal_handlers())
+
         # Children
-        gtk_widget = widget.gtk_widget
-        if not isinstance(gtk_widget, gtk.Container):
+        widget = gadget.widget
+        if not isinstance(widget, gtk.Container):
             return node
 
         # We're not writing children when we have a constructor set
-        if widget.constructor:
+        if gadget.constructor:
             return node
 
-        children = gtk_widget.get_children()
-        if isinstance(gtk_widget, gtk.Table):
-            table = gtk_widget
+        children = widget.get_children()
+        if isinstance(widget, gtk.Table):
+            table = widget
             def table_sort(a, b):
                 res =  cmp(table.child_get_property(a, 'left-attach'),
                            table.child_get_property(b, 'left-attach'))
@@ -204,117 +222,118 @@ class XMLWriter:
                               table.child_get_property(b, 'top-attach'))
                 return res
             children.sort(table_sort)
-        # Boxes doesn't need to be sorted, they're already in the right order
-            
-        for child_widget in children:
-            child = self._write_child(child_widget)
-            if child:
-                node.appendChild(child)
 
-        widget.maintain_gtk_properties = False
-                    
+        # Boxes doesn't need to be sorted, they're already in the right order
+        for child_widget in children:
+            if isinstance(child_widget, Placeholder):
+                child_node = self._write_placeholder(child_widget)
+            else:
+                # if there is no gadget for this child
+                # we don't save it. If your children are not being
+                # saved you should create a gadget for them
+                # in your Adaptor
+                child_gadget = Gadget.from_widget(child_widget)
+                if child_gadget is None:
+                    continue
+                child_node = self._write_child(child_gadget)
+
+            node.appendChild(child_node)
+
+        gadget.maintain_gtk_properties = False
+
         return node
 
-    def _write_basic_information(self, widget):
-        assert widget.adaptor.type_name
-        assert widget.name, 'widget %r is nameless' % widget.gtk_widget
+    def _write_properties(self, widget_node, gadget, child):
+        properties = prop_registry.get_writable_properties(gadget.widget,
+                                                           child)
+        properties.sort(lambda a, b: cmp(a.name, b.name))
+
+        for prop_type in properties:
+            prop = gadget.get_prop(prop_type.name)
+            value = prop.save()
+            if value == None:
+                continue
+            property_node = self._write_property(prop, value)
+            widget_node.appendChild(property_node)
+
+    def _write_signals(self, widget_node, signals):
+        # <signal name="..." handler="..." after="..." object="..."/>
+        for signals in signals:
+            for signal in signals:
+                signal_node = self._doc.createElement(tags.XML_TAG_SIGNAL)
+                signal_node.setAttribute(tags.XML_TAG_NAME, signal.name)
+                signal_node.setAttribute(tags.XML_TAG_HANDLER, signal.handler)
+                if signal.after:
+                    signal_node.setAttribute(tags.XML_TAG_AFTER, tags.TRUE)
+                if signal.object:
+                    signal_node.setAttribute(tags.XML_TAG_OBJECT, signal.object)
+                widget_node.appendChild(signal_node)
+
+    def _write_basic_information(self, gadget):
+        # <widget class="..." id=" constructor="">
+        assert gadget.adaptor.type_name
+        assert gadget.name, 'gadget %r is nameless' % gadget.widget
         node = self._doc.createElement(tags.XML_TAG_WIDGET)
 
         # If name is set write it to the file
-        if widget.adaptor.name:
-            type_name = widget.adaptor.name
+        if gadget.adaptor.name:
+            type_name = gadget.adaptor.name
         else:
-            type_name = widget.adaptor.type_name
-            
+            type_name = gadget.adaptor.type_name
+
         node.setAttribute(tags.XML_TAG_CLASS, type_name)
-        node.setAttribute(tags.XML_TAG_ID, widget.name)
-        if widget.constructor:
-            node.setAttribute('constructor', widget.constructor)
+        node.setAttribute(tags.XML_TAG_ID, gadget.name)
+        if gadget.constructor:
+            node.setAttribute('constructor', gadget.constructor)
         return node
-        
-    def _write_properties(self, widget, widget_node, child):
-        # Sort them properties, to have a stable order
-        gtk_widget = widget.gtk_widget
-        properties = prop_registry.list(gobject.type_name(gtk_widget),
-                                        gtk_widget.get_parent())
-        properties.sort(lambda a, b: cmp(a.name, b.name))
 
-        # write the properties
-        for prop_type in properties:
-            # don't save the name property since it is saved in the id
-            # attribute of the tag
-            if prop_type.name == 'name':
-                continue
+    def _write_child(self, child_gadget):
+        # <child>
+        #   <child internal-name="foo">
+        # </child>
 
-            # Do not save non-editable gobject properties, eg
-            # GtkWindow::screen
-            # hack?
-            if (prop_type.base_type == gobject.GObject.__gtype__ and
-                not prop_type.editable):
-                continue
+        child_node = self._doc.createElement(tags.XML_TAG_CHILD)
 
-            if not prop_type.persistent:
-                continue
-            
-            # child properties are saved later
-            if prop_type.child != child:
-                continue
+        if child_gadget.internal_name is not None:
+            child_node.setAttribute(tags.XML_TAG_INTERNAL_CHILD,
+                                    child_gadget.internal_name)
 
-            prop = widget.get_prop(prop_type.name)
-
-            child_node = self._write_property(prop)
-            if child_node:
-                widget_node.appendChild(child_node)
-
-    def _write_signals(self, widget, widget_node):
-        for signal_name, handlers in widget.signals.items():
-            for handler in handlers:
-                child = self._doc.createElement(tags.XML_TAG_SIGNAL)
-                child.setAttribute(tags.XML_TAG_NAME, handler['name'])
-                child.setAttribute(tags.XML_TAG_HANDLER, handler['handler'])
-                child.setAttribute(tags.XML_TAG_AFTER,
-                                   handler['after'] \
-                                   and tags.TRUE or tags.FALSE)
-                widget_node.appendChild(child)
-                
-    def _write_child(self, gtk_widget):
-        child_tag = self._doc.createElement(tags.XML_TAG_CHILD)
-        if isinstance(gtk_widget, Placeholder):
-            child = self._doc.createElement(tags.XML_TAG_PLACEHOLDER)
-            child_tag.appendChild(child)
-            # we need to write the packing properties of the placeholder.
-            # otherwise the container gets confused when loading its
-            # children
-            packing = self._write_placeholder_properties(gtk_widget)
-            if packing is not None:
-                child_tag.appendChild(packing)
-            return child_tag
-
-        child_widget = Widget.from_widget(gtk_widget)
-        if child_widget is None:
-            # if there is no GazpachoWidget for this child
-            # we don't save it. If your children are not being
-            # saved you should create a GazpachoWidget for them
-            # in your Adaptor
-            return
-            
-        if child_widget.internal_name is not None:
-            child_tag.setAttribute(tags.XML_TAG_INTERNAL_CHILD,
-                                   child_widget.internal_name)
-
-        child = self._write_widget(child_widget)
-        child_tag.appendChild(child)
+        child = self._write_widget(child_gadget)
+        child_node.appendChild(child)
 
         # Append the packing properties
-        packing = self._doc.createElement(tags.XML_TAG_PACKING)
-        self._write_properties(child_widget, packing, child=True)
+        packing_node = self._doc.createElement(tags.XML_TAG_PACKING)
+        self._write_properties(packing_node, child_gadget, child=True)
 
-        if packing.childNodes:
-            child_tag.appendChild(packing)
+        if packing_node.childNodes:
+            child_node.appendChild(packing_node)
 
-        return child_tag
+        return child_node
+
+    def _write_placeholder(self, widget):
+        # <child>
+        #   <placeholder>
+        # </child>
+
+        child_node = self._doc.createElement(tags.XML_TAG_CHILD)
+        placeholder_node = self._doc.createElement(tags.XML_TAG_PLACEHOLDER)
+        child_node.appendChild(placeholder_node)
+
+        # we need to write the packing properties of the placeholder.
+        # otherwise the container gets confused when loading its
+        # children
+        self._write_placeholder_properties(child_node, widget)
+
+        return child_node
+
+    def _write_placeholder_properties(self, child_node, placeholder):
+        # <packing>
+        #    <property>...</property>
+        #    ....
+        # </packing>
+
+        # XXX: use prop_registry for this.
 
-    def _write_placeholder_properties(self, placeholder):
         parent = placeholder.get_parent()
         # get the non default packing properties
         packing_list = []
@@ -323,11 +342,12 @@ class XMLWriter:
             v = parent.child_get_property(placeholder, prop.name)
             if v != prop.default_value:
                 packing_list.append((prop, v))
-        
+
         if not packing_list:
             return
-        
+
         packing_node = self._doc.createElement(tags.XML_TAG_PACKING)
+
         for prop, value in packing_list:
             prop_node = self._doc.createElement(tags.XML_TAG_PROPERTY)
             prop_name = prop.name.replace('-', '_')
@@ -344,16 +364,13 @@ class XMLWriter:
             prop_node.appendChild(text)
             packing_node.appendChild(prop_node)
 
-        return packing_node
-        
-    def _write_property(self, prop):
-        value = prop.save()
-        
-        # None means it doesn't need to be saved, it already contains
-        # the default value or shouldn't be saved
-        if value == None:
-            return
-        
+        child_node.appendChild(packing_node)
+
+    def _write_property(self, prop, value):
+        # <property name="..."
+        #           context="yes|no"
+        #           translatable="yes|no">...</property>
+
         node = self._doc.createElement(tags.XML_TAG_PROPERTY)
 
         # We should change each '-' by '_' on the name of the property
@@ -372,4 +389,4 @@ class XMLWriter:
         text = self._doc.createTextNode(escape(value))
         node.appendChild(text)
         return node
-    
+
