@@ -24,7 +24,6 @@ import gtk
 
 from gazpacho.choice import enum_to_string, flags_to_string
 from gazpacho.environ import environ
-from gazpacho.loader.custom import adapter_registry
 
 _ = gettext.gettext
 
@@ -55,13 +54,13 @@ class PropertyCustomEditor(object):
 
     def get_editor_widget(self):
         "Return the gtk widget that implements this editor"
-        
-    def update(self, context, gtk_widget, proxy):
+
+    def update(self, context, widget, proxy):
         """
         Called when a widget with this property is selected
 
         @param    context: gazpacho context to get the current app and project
-        @param gtk_widget: the widget we are editing
+        @param widget: the widget we are editing
         @param      proxy: an object that should be used to update the widget.
                            This is what allows the undo/redo mechanism actually
                            works
@@ -74,36 +73,36 @@ class PropRegistry:
 
         # A list of custom properties
         self._custom = {}
-        
+
         # Cache used by _list_properties, list of pspecs
         self._pspec_cache = {}
-        
+
     def add_base_type(self, klass, gtype):
         if not issubclass(klass, PropType):
             raise TypeError("klass needs to be a subclass of PropType")
-        
+
         if gtype in self._bases:
             raise PropertyError("base type %r is already defined" % gtype)
         elif klass.base_type != None:
             raise PropertyError("klass type %r is already registered to %r" %
                                 (klass, klass.base_type))
-        
+
         self._bases[gtype] = klass
         klass.base_type = gtype
-        
+
     def override_property(self, full_name, klass, child=False):
         """
         Create a property for full_name, (type::prop_name)
         If it doesn't exist add a custom property
         @param klass: property class
         @type  klass: subclass of PropType
-        @param full_name: complete name of property 
+        @param full_name: complete name of property
         @type  full_name: string (typename::prop_name)
         """
 
         if not issubclass(klass, PropType):
             raise TypeError("klass needs to be a subclass of PropType")
-        
+
         type_name, prop_name = full_name.split('::')
         pspec = self._get_pspec(type_name, prop_name)
 
@@ -115,7 +114,7 @@ class PropRegistry:
             #raise TypeError("%s exists, use override_simple" % full_name)
             self._set_interval_from_pspec(klass, pspec)
             self._add_type(full_name, klass, pspec)
-        
+
         # Note, that at this point we need to setup the overriden class
         # since we're not going to call the type constructor, so
         # this needs to be in sync with PropMeta.new
@@ -123,11 +122,11 @@ class PropRegistry:
         klass.name = prop_name
         klass.owner_name = type_name
         klass.owner_type = gobject.type_from_name(type_name)
-        
+
         # Do not always override, so a base type can specify it
         if klass.child is None:
             klass.child = child
-        
+
         if not issubclass(klass.editor, PropertyCustomEditor):
             raise TypeError("editor %r for property class %r needs to be a "
                             "subclass of PropertyCustomEditor " % (
@@ -146,7 +145,7 @@ class PropRegistry:
         parent = None
         if child or (base and base.child):
             parent = type_name
-            
+
         pspec = self._get_pspec(type_name, prop_name, parent)
         if not pspec:
             raise TypeError("You can only override existing properties, %s "
@@ -154,12 +153,12 @@ class PropRegistry:
 
         if not base:
             base = CustomProperty
-            
+
         parent = self._get_base_type(pspec.value_type)
         klass = type(full_name, (base, parent), kwargs)
 
         return self._create_type_from_pspec(pspec, klass, child=child)
-        
+
     def list(self, type_name, parent=None):
         """
         List all the properties for type_name
@@ -181,9 +180,9 @@ class PropRegistry:
             for pspec in self._list_child_properties(parent):
                 prop_types.append(self._get_type_from_pspec(pspec, child=True))
             property_types.extend(prop_types)
-            
+
         property_types.extend(self._list_custom(type_name))
-        
+
         for prop_type in property_types[:]:
             if (not prop_type.readable or
                 not prop_type.writable or
@@ -198,7 +197,7 @@ class PropRegistry:
         does not create the property type if it's not already created.
         @param type_name: gtype name of owner
         @type  type_name: string
-        @param prop_name: name of property 
+        @param prop_name: name of property
         @type  prop_name: string
         """
         pspec = self._get_pspec(type_name, prop_name, parent)
@@ -212,21 +211,21 @@ class PropRegistry:
         child = False
         if pspec in self._list_child_properties(parent):
             child = True
-                                                
+
         return self._get_type_from_pspec(pspec, child=child)
-            
+
     def _get_type_from_pspec(self, pspec, child=False):
         full_name = gobject.type_name(pspec.owner_type) + '::' + pspec.name
         if full_name in self._prop_types:
             return self._prop_types[full_name]
-        
+
         return self._create_type_from_pspec(pspec, child=child)
 
     def _get_base_type(self, value_type):
         if gobject.type_is_a(value_type, gobject.TYPE_OBJECT):
             base_type = gobject.TYPE_OBJECT
         elif gobject.type_is_a(value_type, gobject.TYPE_ENUM):
-            base_type = gobject.TYPE_ENUM 
+            base_type = gobject.TYPE_ENUM
         elif gobject.type_is_a(value_type, gobject.TYPE_FLAGS):
             base_type = gobject.TYPE_FLAGS
         elif gobject.type_is_a(value_type, gobject.TYPE_BOXED):
@@ -238,11 +237,11 @@ class PropRegistry:
         if not base:
             raise PropertyError("Can't find base type for %r" % base_type)
         return base
-        
+
     def _create_type_from_pspec(self, pspec, base=None, child=False):
         if not base:
             base = self._get_base_type(pspec.value_type)
-        
+
         readable = pspec.flags & gobject.PARAM_READABLE != 0
         writable = pspec.flags & gobject.PARAM_WRITABLE != 0
         prop_type = PropMeta.new(gobject.type_name(pspec.owner_type),
@@ -265,7 +264,7 @@ class PropRegistry:
         if maximum == -1 and pspec.value_type == gobject.TYPE_UINT:
             maximum = sys.maxint
         prop_type.maximum = maximum
-            
+
         prop_type.minimum = pspec.minimum
         prop_type.defautl = pspec.default_value
 
@@ -286,9 +285,9 @@ class PropRegistry:
         # are overriding existing ones, they need a another storage
         #if prop_name in self._prop_types:
         #    raise TypeError
-            
+
         self._prop_types[prop_name] = prop_type
-    
+
     def _add_custom_type(self, klass, type_name, prop_name, pspec):
         if not type_name in self._custom:
             custom = self._custom[type_name] = []
@@ -323,16 +322,16 @@ class PropRegistry:
                 break
 
             type_name = gobject.type_name(gobject.type_parent(type_name))
-            
+
         return retval
 
     def _get_pspec(self, type_name, prop_name, parent=None):
         for pspec in self._list_properties(type_name):
             if pspec.name == prop_name:
                 return pspec
-            
+
         # List child properties even if we're not specifying a
-        # parent        
+        # parent
         if parent:
             type_name = parent
 
@@ -340,11 +339,11 @@ class PropRegistry:
             for pspec in self._list_child_properties(type_name):
                 if pspec.name == prop_name:
                     return pspec
-        
+
     def _has_property(self, type_name, prop_name):
         "Checks if  type_name has a property with name prop_name"
         return self._get_pspec(type_name, prop_name) != None
-    
+
     def _list_properties(self, type_name):
         "List all properties for type_name, and use a simple cache"
         pspecs = self._pspec_cache.get(type_name, [])
@@ -356,17 +355,17 @@ class PropRegistry:
             except TypeError:
                 raise TypeError("Unknown GType name: %s" % type_name)
             pspecs = list(pspecs)
-            
+
         self._pspec_cache[type_name] = pspecs
 
-        try:
-            parent_type = gobject.type_parent(type_name)
-            parent_pspecs = gobject.list_properties(parent_type)
-        except RuntimeError:
-            return pspecs
-        
+        #try:
+        #    parent_type = gobject.type_parent(type_name)
+        #    parent_pspecs = gobject.list_properties(parent_type)
+        #except RuntimeError:
+        #    return pspecs
+
         return pspecs
-    
+
     def _list_child_properties(self, parent):
         "List all the child properties for parent"
         if not parent:
@@ -374,6 +373,44 @@ class PropRegistry:
 
         return gtk.container_class_list_child_properties(parent)
 
+    def get_writable_properties(self, widget, child):
+        """
+        Lists all the writable properties, eg, the ones we'd like
+        to have saved in a persistent format, eg glade files
+
+        @param widget: a gtk.Widget
+        @param child: True to list child properties, False to omit them
+        """
+
+        all_properties = prop_registry.list(gobject.type_name(widget),
+                                            widget.get_parent())
+
+        properties = []
+        # write the properties
+        for prop_type in all_properties:
+            # don't save the name property since it is saved in the id
+            # attribute of the tag
+            if prop_type.name == 'name':
+                continue
+
+            # Do not save non-editable gobject properties, eg
+            # GtkWindow::screen
+            # hack?
+            if (prop_type.base_type == gobject.GObject.__gtype__ and
+                not prop_type.editable):
+                continue
+
+            if not prop_type.persistent:
+                continue
+
+            # child properties are saved later
+            if prop_type.child != child:
+                continue
+
+            properties.append(prop_type)
+
+        return properties
+
 class PropMeta(type):
     """
     I am a metaclass used for Property types.
@@ -381,18 +418,18 @@ class PropMeta(type):
     is also a class method called new which is used as
     a factory for types which has me as a metaclass.
     """
-    
-    def new(meta, owner_name, name, base, value_type, owner_type,
+
+    def new(mcs, owner_name, name, base, value_type, owner_type,
             readable=True, writable=True, child=False, label=None):
 
-        cls = meta(owner_name + '::' + name, (base,), {})
+        cls = mcs(owner_name + '::' + name, (base,), {})
 
         # This can not be overridden
         cls.name =  name
         cls.type = value_type
         cls.owner_name = owner_name
         cls.owner_type = owner_type
-        
+
         # Be very careful here.
         # Do not overwrite anything, since we want this class
         # to be as transparent as possible.
@@ -417,7 +454,7 @@ class PropMeta(type):
             getter = base._child_get
         else:
             getter = base._get
-            
+
         if not writable:
             setter = PropType._value_not_writable
         elif child:
@@ -438,23 +475,23 @@ class PropMeta(type):
             if not label:
                 label = name.capitalize()
             cls.label = label
-        
+
         return cls
     new = classmethod(new)
-        
-    def __repr__(cls):
+
+    def __repr__(mcs):
         child = ''
-        if cls.child:
+        if mcs.child:
             child = ' (child)'
-        return '<PropType %s%s>' % (cls.__name__, child)
-        
+        return '<PropType %s%s>' % (mcs.__name__, child)
+
 class PropType(object):
     __metaclass__ = PropMeta
     """
     I am the base class for all properties.
     I wrap a GParamSpec (or custom property types), instances
     of me wrap a GObject property.
-    
+
     @cvar name:         name of the property, eg expand
     @cvar owner_name:   owner gobject name: GtkWindow
     @cvar type:         type, GType of property
@@ -475,7 +512,7 @@ class PropType(object):
     @cvar editor:       editor instance, subclass of XXX
     @cvar persistent:   Should we save the property?
     """
-    
+
     name = None
     owner_name = None
     type = None
@@ -496,19 +533,17 @@ class PropType(object):
     editor = PropertyCustomEditor
     persistent = True
     disabled = False
-    
-    def __init__(self, widget):
+
+    def __init__(self, gadget):
         cls = type(self)
         assert cls.name != None
         assert cls.owner_name != None
         assert cls.owner_type != gobject.TYPE_INVALID
-        
-        self.widget = widget
-        self._project = widget.project
-        gobj = widget.gtk_widget
-        self._adapter = adapter_registry.get_adapter(gobj,
-                                                     self._project.get_loader())
-        
+
+        self.gadget = gadget
+        self._project = gadget.project
+        gobj = gadget.widget
+
         # Assert that the owner is of correct type
         if self.child:
             test_object = gobj.get_parent()
@@ -517,21 +552,22 @@ class PropType(object):
         if not gobject.type_is_a(test_object, self.owner_type):
             raise TypeError("Expected an object of type %s, got %r" % (
                 gobject.type_name(self.owner_type), test_object))
-            
+
         self.load()
-        
+
     def get_object(self):
-        return self.widget.gtk_widget
+        return self.gadget.widget
     object = property(get_object)
-    
+
     def load(self):
         parent_name = None
         if self.child:
             parent_name = gobject.type_name(self.object.get_parent())
-            
+
         # use the gtk value if this is an internal child
-        if self.widget.internal_name or self.widget.maintain_gtk_properties:
-            self._initial = self.widget.adaptor.get_default_prop_value(self, parent_name)
+        if self.gadget.internal_name or self.gadget.maintain_gtk_properties:
+            self._initial = self.gadget.adaptor.get_default_prop_value(
+                self, parent_name)
             # we have to initialize _value from the gtk widget
             if self.custom:
                 self._value = self._initial
@@ -541,7 +577,8 @@ class PropType(object):
                     self._value = parent.child_get_property(self.object,
                                                             self.name)
                 else:
-                    # FIXME: This is broke, but needed for GtkRadioButton::group,
+                    # FIXME: This is broke, but needed for
+                    # GtkRadioButton::group,
                     # see tests.test_glade.GladeTest.test_radiobutton
                     try:
                         self._value = self.object.get_property(self.name)
@@ -552,20 +589,21 @@ class PropType(object):
                 self._initial = self.default
                 self.set(self.default)
             else:
-                self._initial = self.widget.adaptor.get_default_prop_value(self, parent_name)
+                self._initial = self.gadget.adaptor.get_default_prop_value(
+                    self, parent_name)
 
     def get_project(self):
         return self._project
-    
+
     def get_object_name(self):
         if isinstance(self.object, gtk.Widget):
             return self.object.name
-        
+
         return repr(self.object)
 
     def _value_not_readable(self):
         raise PropertyError("%s not readable" % self.name)
-    
+
     def _value_not_writable(self, value):
         raise PropertyError("%s not writable" % self.name)
 
@@ -574,13 +612,12 @@ class PropType(object):
             parent = self.object.get_parent()
             return parent.child_get_property(self.object, self.name)
         except TypeError, e:
-            raise TypeError("Failed to get child property from %r for %s "
-                            "(%s:%s): `%s'" % (parent,
-                                               self.object,
-                                                gobject.type_name(self.owner_type),
-                                               self.name,
-                                               e))
-        
+            raise TypeError(
+                "Failed to get child property from %r for %s "
+                "(%s:%s): `%s'" % (parent, self.object,
+                                   gobject.type_name(self.owner_type),
+                                   self.name, e))
+
     def _child_set(self, value):
         try:
             parent = self.object.get_parent()
@@ -590,7 +627,7 @@ class PropType(object):
         except TypeError, e:
             raise TypeError("Failed to set property %s:%s to %s: `%s'" % (
                 gobject.type_name(self.owner_type), self.name, value, e))
-                
+
 
     def _get(self):
         try:
@@ -606,16 +643,16 @@ class PropType(object):
             raise TypeError("Error while setting %s:%s to %s `%s'" % (
                 gobject.type_name(self.owner_type), self.name, value, e))
 
-    def get_default(cls, gobj):
-        if cls.custom:
-            return cls.default
-        
-        if cls.child:
+    def get_default(self, gobj):
+        if self.custom:
+            return self.default
+
+        if self.child:
             parent = gobj.get_parent()
-            return parent.child_get_property(gobj, cls.name)
-            
-        return gobj.get_property(cls.name)
-        
+            return parent.child_get_property(gobj, self.name)
+
+        return gobj.get_property(self.name)
+
     def is_translatable(self):
         """
         Is the property translatable?
@@ -625,7 +662,7 @@ class PropType(object):
         in those cases, just override this method
         """
         return self.translatable
-    
+
     def save(self):
         """
         Marshal the property into a string
@@ -634,14 +671,14 @@ class PropType(object):
         value = self.get()
         if value == None or value == self._initial:
             return
-        
+
         return str(value)
-        
+
     def __repr__(self):
         return '<Property %s:%s value=%r>' % (self.owner_name, self.name,
                                               self.get())
 
-# XXX: This is a really bad name since it conflicts with 
+# XXX: This is a really bad name since it conflicts with
 #      the custom widget
 class CustomProperty(PropType):
     def get(self):
@@ -649,7 +686,7 @@ class CustomProperty(PropType):
 
     def set(self, value):
         self._set(value)
-        
+
     value = property(lambda self: self.get(),
                      lambda self, value: self.set(value))
 
@@ -657,8 +694,10 @@ class CustomProperty(PropType):
 # required for epydoc to run
 if environ.epydoc:
     class FakeRegistry:
-        def add_base_type(*args, **kwargs): pass
-        def override_property(*args, **kwargs): pass
+        def add_base_type(self, *args, **kwargs):
+            pass
+        def override_property(self, *args, **kwargs):
+            pass
     prop_registry = FakeRegistry()
     custom = object
 else:
@@ -669,7 +708,7 @@ class CustomChildProperty(custom):
     child = True
     def get(self):
         return self._child_get()
-    
+
     def set(self, value):
         self._child_set(value)
 
@@ -684,10 +723,10 @@ class TransparentProperty(custom):
     def __init__(self, *args):
         super(TransparentProperty, self).__init__(*args)
         self._value = self._initial
-        
+
     def get(self):
         return self._value
-    
+
     def set(self, value):
         self._value = value
 
@@ -714,7 +753,7 @@ class ObjectType(PropType):
         value = self.get()
         if value:
             return value.get_name()
-    
+
 prop_registry.add_base_type(ObjectType, gobject.TYPE_OBJECT)
 
 #
@@ -737,10 +776,10 @@ class BaseNumericType(PropType):
     # SpinButton
     # acceleration factor
     climb_rate = 1
-    
+
     # number of decimals
     digits = 0
-    
+
     def get_adjustment(cls):
         #if not cls.minimum <= cls.default <= cls.maximum:
         #    raise TypeError(
@@ -748,14 +787,14 @@ class BaseNumericType(PropType):
         #        "than maximum (%r <= %r <= %r) for %s::%s (%r)" % (
         #        cls.minimum, cls.default, cls.maximum,
         #        gobject.type_name(cls.owner_type), cls.name, cls.type))
-            
+
         return gtk.Adjustment(value=cls.default,
                               lower=cls.minimum,
                               upper=cls.maximum,
                               step_incr=cls.step_increment,
                               page_incr=cls.page_increment)
     get_adjustment = classmethod(get_adjustment)
-    
+
 class BaseFloatType(BaseNumericType):
     """
     Base float type, used by float and double
@@ -764,7 +803,7 @@ class BaseFloatType(BaseNumericType):
     minimum = -10.0**308
     maximum = 10.0**308
     digits = 2
-    
+
 class IntType(BaseNumericType):
     minimum = -int(2**31-1)
     maximum = int(2**31-1)
@@ -793,18 +832,18 @@ class EnumType(PropType):
         choices = []
         if not hasattr(cls.enum_class, '__enum_values__'):
             raise UnsupportedProperty
-        
-        for value, enum in cls.enum_class.__enum_values__.items():
+
+        for enum in cls.enum_class.__enum_values__.values():
             choices.append((enum.value_nick, enum))
         choices.sort()
         return choices
     get_choices = classmethod(get_choices)
-    
+
     def save(self):
         value = self.get()
         if value == self._initial:
             return
-        
+
         return enum_to_string(value, enum=self.enum_class)
 
 prop_registry.add_base_type(EnumType, gobject.TYPE_ENUM)
@@ -822,13 +861,13 @@ class FlagsType(PropType):
         flags.sort()
         return flags
     get_choices = classmethod(get_choices)
-    
+
     def save(self):
         value = self.get()
         if value == self._initial or value == 0:
             return
         return flags_to_string(value, flags=self.flags_class)
-    
+
 prop_registry.add_base_type(FlagsType, gobject.TYPE_FLAGS)
 
 class BoxedType(PropType):
@@ -842,7 +881,7 @@ prop_registry.add_base_type(CharType, gobject.TYPE_CHAR)
 class AdjustmentType(TransparentProperty):
     def save(self):
         adj = self.get()
-        s = [] 
+        s = []
         for value in (adj.value, adj.lower, adj.upper,
                       adj.step_increment, adj.page_increment,
                       adj.page_size):
@@ -856,14 +895,14 @@ prop_registry.add_base_type(AdjustmentType, gtk.Adjustment)
 
 class ProxiedProperty(TransparentProperty):
     target_name = None
-    
+
     def get_target(self):
         raise NotImplementedError
-    
+
     def get(self):
         target = self.get_target()
         return target.get_property(self.target_name)
-        
+
     def set(self, value):
         target = self.get_target()
         return target.set_property(self.target_name, value)
@@ -896,6 +935,6 @@ if __name__ == '__main__':
     #    prop = ptype(box, None)
     #    if prop.readable:
     #        print ptype, prop.value
-        
+
     raise SystemExit
 
